package org.processmining.plugins.plg.generator.process;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.processmining.framework.annotations.AuthoredType;
import org.processmining.plugins.plg.config.BasicPluginConfiguration;

/**
 * Configuration of a collection of processes generator
 * 
 * @author Andrea Burattin
 */
@AuthoredType(
	typeName = "PLG Process collection configuration",
	affiliation = BasicPluginConfiguration.AFFILIATION,
	author = BasicPluginConfiguration.AUTHOR,
	email = BasicPluginConfiguration.EMAIL
)
public class PLGProcessCollectionGeneratorConfiguration {

	private Date creationDate = new Date();
	private String collectionName = "";
	private PLGProcessGeneratorConfiguration singleProcess = new PLGProcessGeneratorConfiguration(0, null, 0, null, 0, 0, 0, 0, 0, 0, null, null);
	private int instancesNo = -1;
	
	
	/**
	 * @param name
	 * @param singleProcess
	 * @param instancesNo
	 */
	public PLGProcessCollectionGeneratorConfiguration(
			String name,
			PLGProcessGeneratorConfiguration singleProcess,
			int instancesNo) {
		setCollectionName(name);
		setSingleProcess(singleProcess);
		setInstancesNo(instancesNo);
		creationDate = new Date();
	}
	
	
	/**
	 * @param instancesNo
	 */
	public void setInstancesNo(int instancesNo) {
		if (instancesNo < 1) {
			instancesNo = 1;
		}
		this.instancesNo = instancesNo;
	}
	
	
	/**
	 * @return
	 */
	public int getInstancesNo() {
		return instancesNo;
	}
	
	
	/**
	 * @param singleProcess
	 */
	public void setSingleProcess(PLGProcessGeneratorConfiguration singleProcess) {
		this.singleProcess = singleProcess;
	}
	
	
	/**
	 * @return
	 */
	public PLGProcessGeneratorConfiguration getSingleProcess() {
		return singleProcess;
	}
	
	
	/**
	 * @param name
	 */
	public void setCollectionName(String name) {
		this.collectionName = name.trim();
	}
	
	
	/**
	 * @return
	 */
	public String getCollectionName() {
		if (this.collectionName.length() > 0) {
			return this.collectionName;
		} else {
			return "Process Log Colletion" + new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(new Date());
		}
	}
	
	
	/**
	 * 
	 * @param creationDate
	 */
	public void setCreationDate(Date creationDate) {
		this.creationDate = creationDate;
	}
	
	
	/**
	 * @return
	 */
	public Date getCreationDate() {
		return this.creationDate;
	}
}
