package org.processmining.plugins.plg.generator.process;

import org.processmining.framework.annotations.AuthoredType;
import org.processmining.plugins.plg.config.BasicPluginConfiguration;
import org.processmining.plugins.plg.logic.models.PlgParameters;
import org.processmining.plugins.plg.logic.models.distributions.ProbabilityDistribution;


/**
 * Configuration of a single process generator
 * 
 * @author Andrea Burattin
 */
@AuthoredType(
	typeName = "PLG Process configuration",
	affiliation = BasicPluginConfiguration.AFFILIATION,
	author = BasicPluginConfiguration.AUTHOR,
	email = BasicPluginConfiguration.EMAIL
)
public class PLGProcessGeneratorConfiguration extends PlgParameters {

	/**
	 * Class constructor. To build a parameters configuration, all parameters
	 * must be given
	 * 
	 * @param ANDBranches the maximum number of AND branches (must be > 1)
	 * @param ANDbranchesDistribution
	 * @param XORBranches the maximum number of XOR branches (must be > 1)
	 * @param XORbranchesDistribution
	 * @param loopPercent the loop probability (must be >= 0 and <= 100)
	 * @param singleActivityPercent the probability of single activity (must 
	 * be >= 0 and <= 100)
	 * @param sequencePercent he probability of sequence activity (must be 
	 * >= 0 and <= 100)
	 * @param ANDPercent the probability of AND split-join (must be >= 0 and 
	 * <= 100)
	 * @param XORPercent the probability of XOR split-join (must be >= 0 and 
	 * <= 100)
	 * @param emptyPercent the probability of an empty pattern (must be >= 0
	 * and <= 100)
	 * @param deep the maximum network deep
	 * @param andExecDistribution 
	 * @param xorExecDistribution 
	 */
	public PLGProcessGeneratorConfiguration(int ANDBranches,
			ProbabilityDistribution ANDbranchesDistribution,
			int XORBranches,
			ProbabilityDistribution XORbranchesDistribution,
			int loopPercent, int singleActivityPercent, int sequencePercent,
			int ANDPercent, int XORPercent, int deep,
			ProbabilityDistribution andExecDistribution,
			ProbabilityDistribution xorExecDistribution) {
		super(ANDBranches, ANDbranchesDistribution, XORBranches,
				XORbranchesDistribution, loopPercent, singleActivityPercent,
				sequencePercent, ANDPercent, XORPercent, deep, andExecDistribution,
				xorExecDistribution);
	}
	
}
