package org.processmining.plugins.plg.gui;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.DecimalFormat;
import java.text.ParseException;

import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.text.NumberFormatter;

import org.deckfour.uitopia.ui.util.BrowserLauncher;

import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * A collection of user interface utilities
 * 
 * @author Andrea Burattin
 */
public class GUIUtils {

	/**
	 * This method creates a new JLabel correctly formatted
	 * 
	 * @param s the string to be inserted into the label
	 * @return a component with the required widget
	 */
	public static JLabel prepareLabel(String s) {
		return prepareLabel(s, SwingConstants.LEFT);
	}
	

	/**
	 * This method creates a new JLabel correctly formatted
	 * 
	 * @param s the string to be inserted into the label
	 * @param alignment the alignment
	 * @return a component with the required widget
	 */
	public static JLabel prepareLabel(String s, int alignment) {
		JLabel l = SlickerFactory.instance().createLabel(s);
		l.setHorizontalAlignment(alignment);
		l.setForeground(new Color(40,40,40));
		return l;
	}
	
	
	/**
	 * This method creates a new JLabel correctly formatted
	 * 
	 * @param i the string to be inserted into the label
	 * @return a component with the required widget
	 */
	public static JLabel prepareLabel(int i) {
		return prepareLabel(i, SwingConstants.LEFT);
	}
	
	
	/**
	 * This method creates a new JLabel correctly formatted
	 * 
	 * @param i the string to be inserted into the label
	 * @param alignment the alignment
	 * @return a component with the required widget
	 */
	public static JLabel prepareLabel(int i, int alignment) {
		Integer value = i;
		JLabel l = SlickerFactory.instance().createLabel(value.toString());
		l.setHorizontalAlignment(alignment);
		l.setForeground(new Color(40,40,40));
		return l;
	}
	
	
	/**
	 * This method creates a new JLabel correctly formatted in order to be a
	 * frame title. The text will be inserted into a gray box.
	 * 
	 * @param s the string to be inserted into the label
	 * @return a component with the required widget
	 */
	public static JPanel prepareBorderedTitle(String s) {
		SlickerFactory factory = SlickerFactory.instance();
		JPanel title = factory.createRoundedPanel(15, new Color(40,40,40));
		title.setBounds(0, 0, 780, 40);
		JLabel labelTitle = factory.createLabel(s);
		labelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		labelTitle.setForeground(Color.gray);
		labelTitle.setBounds(10, 5, 760, 30);
		title.setLayout(null);
		title.add(labelTitle);
		return title;
	}
	
	
	/**
	 * This method creates a new JLabel correctly formatted in order to be a
	 * frame title
	 * 
	 * @param s the string to be inserted into the label
	 * @return a component with the required widget
	 */
	public static JLabel prepareTitle(String s) {
		JLabel labelTitle = SlickerFactory.instance().createLabel(s);
		labelTitle.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD, 18));
		labelTitle.setForeground(Color.darkGray);
		labelTitle.setBorder(new EmptyBorder(5, 0, 10, 0));
		return labelTitle;
	}
	
	
	/**
	 * This method creates a new text field for receiving text as input
	 * 
	 * @param initialContent the initial content of the text field
	 * @return the formatted text field
	 */
	public static JTextField prepareTextField(String initialContent) {
		JTextField tmp = new JTextField(initialContent);
		tmp.setColumns(8);
		tmp.setText(initialContent);
		tmp.setBackground(Color.DARK_GRAY);
		tmp.setForeground(Color.LIGHT_GRAY);
		tmp.setCaretColor(Color.GRAY);
		tmp.setBorder(javax.swing.BorderFactory.createLineBorder(Color.DARK_GRAY, 2));
		return tmp;
	}
	
	
	/**
	 * This method creates a new text field for receiving integer (up to 5
	 * digits) as input
	 * 
	 * @param initialContent the initial content of the text field
	 * @return the formatted text field
	 */
	public static JTextField prepareIntegerField(int initialContent) {
		DecimalFormat df = new DecimalFormat("#####");
		NumberFormatter nf = new NumberFormatter(df) {
			private static final long serialVersionUID = 1L;
			public String valueToString(Object iv) throws ParseException {
				if ((iv == null) || (((Integer) iv).intValue() == -1)) {
					return "";
				} else {
					return super.valueToString(iv);
				}
			}
			public Object stringToValue(String text) throws ParseException {
				if ("".equals(text)) {
					return null;
				}
				return super.stringToValue(text);
			}
		};
		nf.setMinimum(0);
		nf.setMaximum(65534);
		nf.setValueClass(Integer.class);
		
		JTextField tmp = new JFormattedTextField(nf);
		tmp.setColumns(5);
		tmp.setText(new Integer(initialContent).toString());
		tmp.setBackground(Color.darkGray);
		tmp.setForeground(Color.gray);
		tmp.setCaretColor(Color.gray);
		tmp.setBorder(javax.swing.BorderFactory.createLineBorder(Color.darkGray, 2));
		return tmp;
	}
	
	/**
	 * The credit labels
	 */
	public static JLabel credits = GUIUtils.prepareLabel("<html>" +
			"A.Burattin, A.Sperduti. \"<i>PLG: a Framework<br>" +
			"for the Generation of Business Process Models<br>" +
			"and their Execution Logs</i>\". BPI Workshop 2011.<br>" +
			"DOI: 10.1007/978-3-642-20511-8_20.</html>");
	static {
		credits.setForeground(Color.DARK_GRAY);
		credits.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		credits.addMouseListener(new MouseListener() {
			public void mouseReleased(MouseEvent e) {}
			public void mousePressed(MouseEvent e) {}
			public void mouseExited(MouseEvent e) {
				credits.setForeground(Color.DARK_GRAY);
			}
			public void mouseEntered(MouseEvent e) {
				credits.setForeground(Color.GRAY);
			}
			public void mouseClicked(MouseEvent e) {
				BrowserLauncher.openURL("http://plg.processmining.it");
			}
		});
	}
}
