package org.processmining.plugins.plg.gui;

import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import org.processmining.plugins.plg.generator.log.PLGLogCollectionGeneratorConfiguration;
import org.processmining.plugins.plg.generator.log.PLGLogGeneratorConfiguration;

import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * Panel for the configuration of the log generator. This class realizes the
 * panel for the configuration of the parameters for the generation of logs.
 *
 * @author Andrea Burattin
 */
public class LogGeneratorParameters extends JPanel {

	private static final long serialVersionUID = 3175143610519917807L;
	private JTextField paramCollectionName, paramLogsPerProcess, paramTraces;
	private NiceIntegerSlider sliderNoise, sliderDuration;


	public LogGeneratorParameters() {
		init();
	}


	/**
	 * Method to obtain the configuration for the collection of processes, given
	 * the current setting of the configuration dialog
	 *
	 * @return the configuration object for the collection of models
	 */
	public PLGLogCollectionGeneratorConfiguration getSettings() {
		String tmpName = paramCollectionName.getText();
		Integer tmpLogs = Integer.parseInt(paramLogsPerProcess.getText());
		Integer tmpTraces = Integer.parseInt(paramTraces.getText());
		Integer tmpNoise = sliderNoise.getValue();
		Integer tmpInterval = sliderDuration.getValue();

		PLGLogGeneratorConfiguration tmpSingleLog = new PLGLogGeneratorConfiguration(tmpTraces, tmpInterval, tmpNoise);
		PLGLogCollectionGeneratorConfiguration tmp = new PLGLogCollectionGeneratorConfiguration(tmpName, tmpSingleLog, tmpLogs);

		return tmp;
	}


	/**
	 * The GUI initializer
	 */
	private void init() {
		SlickerFactory factory = SlickerFactory.instance();

		JLabel l00 = GUIUtils.prepareLabel("Collection name:", SwingConstants.RIGHT);
		paramCollectionName = GUIUtils.prepareTextField("Logs collection " + new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(new Date()));
		JLabel l01 = GUIUtils.prepareLabel("Number of logs per process:", SwingConstants.RIGHT);
		paramLogsPerProcess = GUIUtils.prepareIntegerField(10);
		JLabel l02 = GUIUtils.prepareLabel("Number of traces per log:", SwingConstants.RIGHT);
		paramTraces = GUIUtils.prepareIntegerField(100);
		JLabel l03 = GUIUtils.prepareLabel("Percent of noise", SwingConstants.RIGHT);
		sliderNoise = factory.createNiceIntegerSlider("", 1, 100, 5, Orientation.HORIZONTAL);
		JLabel l04 = GUIUtils.prepareLabel("Percent of activities with duration", SwingConstants.RIGHT);
		sliderDuration = factory.createNiceIntegerSlider("", 1, 100, 100, Orientation.HORIZONTAL);

		setLayout(null);
		add(l00); add(paramCollectionName);
		add(l01); add(paramLogsPerProcess);
		add(l02); add(paramTraces);
		add(l03); add(sliderNoise);
		add(l04); add(sliderDuration);

		l00.setBounds(0,  10, 225, 20); paramCollectionName.setBounds(232,  10, 548, 20);
		l01.setBounds(0,  40, 225, 20); paramLogsPerProcess.setBounds(232,  40,  50, 20);
		l02.setBounds(0,  70, 225, 20);         paramTraces.setBounds(232,  70,  50, 20);
		l03.setBounds(0, 100, 220, 20);         sliderNoise.setBounds(222, 100, 558, 20);
		l04.setBounds(0, 130, 220, 20);      sliderDuration.setBounds(222, 130, 558, 20);

		validate();
		repaint();
	}
}
