package org.processmining.plugins.plg.gui;


import java.awt.CardLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.processmining.plugins.plg.generator.process.PLGProcessCollectionGeneratorConfiguration;
import org.processmining.plugins.plg.generator.process.PLGProcessGeneratorConfiguration;
import org.processmining.plugins.plg.gui.widget.DistributionViewer;
import org.processmining.plugins.plg.logic.models.distributions.ProbabilityDistribution;
import org.processmining.plugins.plg.logic.models.distributions.ProbabilityDistribution.DISTRIBUTION;

import com.fluxicon.slickerbox.components.NiceIntegerSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * Panel for the configuration of the process generator. This class realizes the
 * panel for the configuration of the parameters for the generation of
 * processes.
 *
 * @author Andrea Burattin
 */
public class ProcessGeneratorParameters extends JPanel {

	private static final long serialVersionUID = -4740559384301848969L;
	private SlickerTabbedPane container;
	private JPanel panelProperties, panelProbabilities, panelBranchesAND, panelBranchesXOR;
	private JPanel labelProperties, labelProbabilities, labelBranchesAND, labelBranchesXOR;
	private JTextField paramCollectionName;
	private NiceIntegerSlider sliderCardinality;
	private NiceIntegerSlider sliderNested, sliderLoop, sliderSingleAct, sliderSeq, sliderAND, sliderXOR;
	private NiceIntegerSlider sliderANDBranches, sliderXORBranches;
	@SuppressWarnings("rawtypes")
	private JComboBox comboANDBranches, comboANDSorting,
	                  comboXORBranches, comboXORSorting;
	private DistributionViewer distribANDBranches, distribANDSorting,
	                           distribXORBranches, distribXORSorting;
	private JPanel panelANDParaBranches, panelANDParaSorting,
	               panelXORParaBranches, panelXORParaSorting;
	private JPanel panelANDParaBranchesBeta, panelANDParaSortingBeta,
	               panelXORParaBranchesBeta, panelXORParaSortingBeta;
	private JTextField	paramANDAlphaBranches, paramANDBetaBranches,
						paramANDAlphaSorting, paramANDBetaSorting,
						paramXORAlphaBranches, paramXORBetaBranches,
						paramXORAlphaSorting, paramXORBetaSorting;
	private JButton buttonResetProbabilities;


	public ProcessGeneratorParameters() {
		init();
	}


	/**
	 * Method to obtain the configuration for the collection of processes, given
	 * the current setting of the configuration dialog
	 *
	 * @return the configuration object for the collection of models
	 */
	public PLGProcessCollectionGeneratorConfiguration getSettings() {
		// temporary objects
		String currDistrVal = null;
		ProbabilityDistribution.DISTRIBUTION currDistrib;

		// AND branches generation
		currDistrVal = (String)comboANDBranches.getSelectedItem();
		currDistrib = ProbabilityDistribution.nameToDistribution(currDistrVal);
		ProbabilityDistribution ANDbranchesDistribution = ProbabilityDistribution.normalDistributionFactory();
		if (currDistrib.equals(DISTRIBUTION.UNIFORM)) {
			ANDbranchesDistribution = ProbabilityDistribution.uniformDistributionFactory();
		}
		else if (currDistrib.equals(DISTRIBUTION.BETA)) {
			double alpha = Double.parseDouble(paramANDAlphaBranches.getText());
			double beta = Double.parseDouble(paramANDBetaBranches.getText());
			alpha = (alpha < 0? 0 : alpha);
			beta = (beta < 0? 0 : beta);
			ANDbranchesDistribution = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
		}

		// AND logs generation
		currDistrVal = (String)comboANDSorting.getSelectedItem();
		currDistrib = ProbabilityDistribution.nameToDistribution(currDistrVal);
		ProbabilityDistribution andExecDistribution = ProbabilityDistribution.normalDistributionFactory();
		if (currDistrib.equals(DISTRIBUTION.UNIFORM)) {
			andExecDistribution = ProbabilityDistribution.uniformDistributionFactory();
		}
		else if (currDistrib.equals(DISTRIBUTION.BETA)) {
			double alpha = Double.parseDouble(paramANDAlphaSorting.getText());
			double beta = Double.parseDouble(paramANDBetaSorting.getText());
			alpha = (alpha < 0? 0 : alpha);
			beta = (beta < 0? 0 : beta);
			andExecDistribution = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
		}

		// XOR branches generation
		currDistrVal = (String)comboXORBranches.getSelectedItem();
		currDistrib = ProbabilityDistribution.nameToDistribution(currDistrVal);
		ProbabilityDistribution XORbranchesDistribution = ProbabilityDistribution.normalDistributionFactory();
		if (currDistrib.equals(DISTRIBUTION.UNIFORM)) {
			XORbranchesDistribution = ProbabilityDistribution.uniformDistributionFactory();
		}
		else if (currDistrib.equals(DISTRIBUTION.BETA)) {
			double alpha = Double.parseDouble(paramXORAlphaBranches.getText());
			double beta = Double.parseDouble(paramXORBetaBranches.getText());
			alpha = (alpha < 0? 0 : alpha);
			beta = (beta < 0? 0 : beta);
			XORbranchesDistribution = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
		}

		// XOR log generation
		currDistrVal = (String)comboXORBranches.getSelectedItem();
		currDistrib = ProbabilityDistribution.nameToDistribution(currDistrVal);
		ProbabilityDistribution xorExecDistribution = ProbabilityDistribution.normalDistributionFactory();
		if (currDistrib.equals(DISTRIBUTION.UNIFORM)) {
			xorExecDistribution = ProbabilityDistribution.uniformDistributionFactory();
		}
		else if (currDistrib.equals(DISTRIBUTION.BETA)) {
			double alpha = Double.parseDouble(paramXORAlphaSorting.getText());
			double beta = Double.parseDouble(paramXORBetaSorting.getText());
			alpha = (alpha < 0? 0 : alpha);
			beta = (beta < 0? 0 : beta);
			xorExecDistribution = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
		}

		// single process configuration
		PLGProcessGeneratorConfiguration singleProcessConfig =
			new PLGProcessGeneratorConfiguration(
					sliderANDBranches.getValue(), ANDbranchesDistribution,
					sliderXORBranches.getValue(), XORbranchesDistribution,
					sliderLoop.getValue(),
					sliderSingleAct.getValue(),
					sliderSeq.getValue(),
					sliderAND.getValue(),
					sliderXOR.getValue(),
					sliderNested.getValue(),
					andExecDistribution, xorExecDistribution);

		// number of processes to generate
		int instancesNo = sliderCardinality.getValue();

		// final object
		return new PLGProcessCollectionGeneratorConfiguration(paramCollectionName.getText(), singleProcessConfig, instancesNo);
	}


	/**
	 * The GUI initializer
	 */
	private void init() {
		SlickerFactory factory = SlickerFactory.instance();
		container = factory.createTabbedPane("",
				Color.gray, new Color(40,40,40),
				new Color(220, 220, 220, 180));

		// PROPERTIES PANEL ---------------------------------------------------
		panelProperties = factory.createRoundedPanel(0, Color.gray);
		labelProperties = GUIUtils.prepareBorderedTitle("Collection properties");

		JLabel l00 = GUIUtils.prepareLabel("Collection name:", SwingConstants.RIGHT);
		paramCollectionName = GUIUtils.prepareTextField("Processes collection " + new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(new Date()));
		JLabel l01 = GUIUtils.prepareLabel("Number of processes to generate", SwingConstants.RIGHT);
		sliderCardinality = factory.createNiceIntegerSlider("", 1, 100, 1, Orientation.HORIZONTAL);

		panelProperties.setLayout(null);
		panelProperties.add(labelProperties);
		panelProperties.add(l00);
		panelProperties.add(paramCollectionName);
		panelProperties.add(l01);
		panelProperties.add(sliderCardinality);

		panelProperties.setBounds(0, 0, 780, 90);
		l00.setBounds(20, 60, 225, 20); paramCollectionName.setBounds(250, 60, 510, 20);
		l01.setBounds(20, 90, 220, 20);   sliderCardinality.setBounds(242, 90, 518, 20);

		// DISTRIBUTION PANEL --------------------------------------------------

		sliderNested = factory.createNiceIntegerSlider("", 1, 10, 3, Orientation.HORIZONTAL);
		sliderLoop = factory.createNiceIntegerSlider("", 0, 100, 20, Orientation.HORIZONTAL);
		sliderSingleAct = factory.createNiceIntegerSlider("", 0, 100, 50, Orientation.HORIZONTAL);
		sliderSeq = factory.createNiceIntegerSlider("", 0, 100, 50, Orientation.HORIZONTAL);
		sliderAND = factory.createNiceIntegerSlider("", 0, 100, 50, Orientation.HORIZONTAL);
		sliderXOR = factory.createNiceIntegerSlider("", 0, 100, 50, Orientation.HORIZONTAL);

		panelProbabilities = factory.createRoundedPanel(0, Color.gray);
		labelProbabilities = GUIUtils.prepareBorderedTitle("Basic pattern percentages");

		buttonResetProbabilities = factory.createButton("Reset percentages");

		JLabel l1 = GUIUtils.prepareLabel("Loop percentage", SwingConstants.RIGHT);
		JLabel l2 = GUIUtils.prepareLabel("Single activity percentage", SwingConstants.RIGHT);
		JLabel l3 = GUIUtils.prepareLabel("Sequence percentage", SwingConstants.RIGHT);
		JLabel l4 = GUIUtils.prepareLabel("AND split-join percentage", SwingConstants.RIGHT);
		JLabel l5 = GUIUtils.prepareLabel("XOR split-join percentage", SwingConstants.RIGHT);
		JLabel l6 = GUIUtils.prepareLabel("Maximum nested patterns", SwingConstants.RIGHT);

		panelProbabilities.setLayout(null);
		panelProbabilities.add(labelProbabilities);
		panelProbabilities.add(l6);
		panelProbabilities.add(sliderNested);
		panelProbabilities.add(l1);
		panelProbabilities.add(sliderLoop);
		panelProbabilities.add(l2);
		panelProbabilities.add(sliderSingleAct);
		panelProbabilities.add(l3);
		panelProbabilities.add(sliderSeq);
		panelProbabilities.add(l4);
		panelProbabilities.add(sliderAND);
		panelProbabilities.add(l5);
		panelProbabilities.add(sliderXOR);
		panelProbabilities.add(buttonResetProbabilities);

		panelProbabilities.setBounds(0, 110, 780, 285);
		l6.setBounds(20,  60, 170, 20);    sliderNested.setBounds(192,  60, 568, 20);
		l1.setBounds(20, 110, 170, 20);      sliderLoop.setBounds(192, 110, 568, 20);
		l2.setBounds(20, 140, 170, 20); sliderSingleAct.setBounds(192, 140, 568, 20);
		l3.setBounds(20, 170, 170, 20);       sliderSeq.setBounds(192, 170, 568, 20);
		l4.setBounds(20, 200, 170, 20);       sliderAND.setBounds(192, 200, 568, 20);
		l5.setBounds(20, 230, 170, 20);       sliderXOR.setBounds(192, 230, 568, 20);
		buttonResetProbabilities.setBounds(560, 270, 200, 25);

		buttonResetProbabilities.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				sliderLoop.setValue(20);
				sliderSingleAct.setValue(50);
				sliderSeq.setValue(50);
				sliderAND.setValue(50);
				sliderXOR.setValue(50);
			}
		});

		// BRANCHES PANEL ------------------------------------------------------
		// -------------- AND --------------------------------------------------
		panelBranchesAND = factory.createRoundedPanel(0, Color.gray);
		labelBranchesAND = GUIUtils.prepareBorderedTitle("AND branches configuration");
		sliderANDBranches = factory.createNiceIntegerSlider("", 2, 10, 5, Orientation.HORIZONTAL);
		comboANDBranches = factory.createComboBox(new String[]{
				ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL),
				ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA),
				ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM)});
		distribANDBranches = new DistributionViewer();
		comboANDSorting = factory.createComboBox(new String[]{
				ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL),
				ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA),
				ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM)});
		distribANDSorting = new DistributionViewer();

		JLabel l10 = GUIUtils.prepareLabel("Max number of split-join", SwingConstants.RIGHT);
		JLabel l11 = GUIUtils.prepareLabel("No. of branches distribution:", SwingConstants.RIGHT);
		JLabel l12 = GUIUtils.prepareLabel("Sorting probability distrbution:", SwingConstants.RIGHT);

		panelANDParaBranchesBeta = factory.createRoundedPanel(0, Color.gray);
		panelANDParaSortingBeta = factory.createRoundedPanel(0, Color.gray);

		paramANDAlphaBranches = new JFormattedTextField(NumberFormat.getInstance());
		paramANDBetaBranches = new JFormattedTextField(NumberFormat.getInstance());
		panelANDParaBranches = preparePanelBetaParams(distribANDBranches, paramANDAlphaBranches, paramANDBetaBranches, panelANDParaBranchesBeta, factory);
		paramANDAlphaSorting = new JFormattedTextField(NumberFormat.getInstance());
		paramANDBetaSorting = new JFormattedTextField(NumberFormat.getInstance());
		panelANDParaSorting = preparePanelBetaParams(distribANDSorting, paramANDAlphaSorting, paramANDBetaSorting, panelANDParaSortingBeta, factory);

		panelBranchesAND.setLayout(null);
		panelBranchesAND.add(labelBranchesAND);
		panelBranchesAND.add(l10);
		panelBranchesAND.add(sliderANDBranches);
		panelBranchesAND.add(l11);
		panelBranchesAND.add(comboANDBranches);
		panelBranchesAND.add(distribANDBranches);
		panelBranchesAND.add(l12);
		panelBranchesAND.add(comboANDSorting);
		panelBranchesAND.add(distribANDSorting);
		panelBranchesAND.add(panelANDParaBranches);
		panelBranchesAND.add(panelANDParaSorting);

		panelBranchesAND.setBounds(0, 405, 780, 250);
		l10.setBounds(20, 60, 200, 20); sliderANDBranches.setBounds(220, 60, 540, 20);
		l11.setBounds(20, 90, 205, 20); comboANDBranches.setBounds(230, 90, 535, 25);
		panelANDParaBranches.setBounds(245, 125, 210, 85);
		distribANDBranches.setBounds(550, 125, 210, 85);
		l12.setBounds(20, 230, 205, 20); comboANDSorting.setBounds(230, 230, 535, 25);
		panelANDParaSorting.setBounds(245, 256, 210, 85);
		distribANDSorting.setBounds(550, 265, 210, 85);

		updateDistribution(distribANDBranches, sliderANDBranches.getValue());
		updateDistribution(distribANDSorting, sliderANDBranches.getValue());

		sliderANDBranches.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				updateDistribution(distribANDBranches, sliderANDBranches.getValue());
				updateDistribution(distribANDSorting, sliderANDBranches.getValue());
			}
		});
		comboANDBranches.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                comboANDBranchesChanged(evt);
            }
        });
		comboANDSorting.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                comboANDSortingChanged(evt);
            }
        });

		// -------------- XOR --------------------------------------------------
		panelBranchesXOR = factory.createRoundedPanel(150, Color.gray);
		labelBranchesXOR = GUIUtils.prepareBorderedTitle("XOR branches configuration");
		sliderXORBranches = factory.createNiceIntegerSlider("", 2, 10, 5, Orientation.HORIZONTAL);
		comboXORBranches = factory.createComboBox(new String[]{
				ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL),
				ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA),
				ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM)});
		distribXORBranches = new DistributionViewer();
		comboXORSorting = factory.createComboBox(new String[]{
				ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL),
				ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA),
				ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM)});
		distribXORSorting = new DistributionViewer();

		JLabel l20 = GUIUtils.prepareLabel("Max number of split-join", SwingConstants.RIGHT);
		JLabel l21 = GUIUtils.prepareLabel("No. of branches distribution:", SwingConstants.RIGHT);
		JLabel l22 = GUIUtils.prepareLabel("Sorting probability distrbution:", SwingConstants.RIGHT);

		panelXORParaBranchesBeta = factory.createRoundedPanel(0, Color.gray);
		panelXORParaSortingBeta = factory.createRoundedPanel(0, Color.gray);

		paramXORAlphaBranches = new JFormattedTextField(NumberFormat.getInstance());
		paramXORBetaBranches = new JFormattedTextField(NumberFormat.getInstance());
		panelXORParaBranches = preparePanelBetaParams(distribXORBranches, paramXORAlphaBranches, paramXORBetaBranches, panelXORParaBranchesBeta, factory);
		paramXORAlphaSorting = new JFormattedTextField(NumberFormat.getInstance());
		paramXORBetaSorting = new JFormattedTextField(NumberFormat.getInstance());
		panelXORParaSorting = preparePanelBetaParams(distribXORSorting, paramXORAlphaSorting, paramXORBetaSorting, panelXORParaSortingBeta, factory);

		panelBranchesXOR.setLayout(null);
		panelBranchesXOR.add(labelBranchesXOR);
		panelBranchesXOR.add(l20);
		panelBranchesXOR.add(sliderXORBranches);
		panelBranchesXOR.add(l21);
		panelBranchesXOR.add(comboXORBranches);
		panelBranchesXOR.add(distribXORBranches);
		panelBranchesXOR.add(l22);
		panelBranchesXOR.add(comboXORSorting);
		panelBranchesXOR.add(distribXORSorting);
		panelBranchesXOR.add(panelXORParaBranches);
		panelBranchesXOR.add(panelXORParaSorting);

		panelBranchesXOR.setBounds(0, 405, 780, 250);
		l20.setBounds(20, 60, 200, 20); sliderXORBranches.setBounds(220, 60, 540, 20);
		l21.setBounds(20, 90, 205, 20); comboXORBranches.setBounds(230, 90, 535, 25);
		panelXORParaBranches.setBounds(245, 125, 210, 85);
		distribXORBranches.setBounds(550, 125, 210, 85);
		l22.setBounds(20, 230, 205, 20); comboXORSorting.setBounds(230, 230, 535, 25);
		panelXORParaSorting.setBounds(245, 256, 210, 85);
		distribXORSorting.setBounds(550, 265, 210, 85);

		updateDistribution(distribXORBranches, sliderXORBranches.getValue());
		updateDistribution(distribXORSorting, sliderXORBranches.getValue());

		sliderXORBranches.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				updateDistribution(distribXORBranches, sliderXORBranches.getValue());
				updateDistribution(distribXORSorting, sliderXORBranches.getValue());
			}
		});
		comboXORBranches.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                comboXORBranchesChanged(evt);
            }
        });
		comboXORSorting.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                comboXORSortingChanged(evt);
            }
        });

		// ADD ALL TO THE MAIN PANEL -------------------------------------------

		setLayout(new java.awt.BorderLayout());
		add(container, java.awt.BorderLayout.CENTER);
		container.addTab("Collection properties", panelProperties);
		container.addTab("Basic pattern probabilities", panelProbabilities);
		container.addTab("AND branches configuration", panelBranchesAND);
		container.addTab("XOR branches configuration", panelBranchesXOR);
		validate();
		repaint();
	}


	private JPanel preparePanelBetaParams(final DistributionViewer distr, final JTextField paramAlphaBranches, final JTextField paramBetaBranches, JPanel panelParaBranchesBeta, SlickerFactory factory) {
		JPanel panelParaBranches = factory.createRoundedPanel(0, Color.gray);
		panelParaBranches.setLayout(new CardLayout());
		panelParaBranches.add(factory.createRoundedPanel(0, Color.gray), ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM));
		panelParaBranches.add(factory.createRoundedPanel(0, Color.gray), ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL));
		panelParaBranches.add(panelParaBranchesBeta, ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA));

		paramAlphaBranches.setText("2");
		paramAlphaBranches.setBackground(Color.darkGray);
		paramAlphaBranches.setForeground(Color.gray);
		paramAlphaBranches.setBorder(javax.swing.BorderFactory.createLineBorder(Color.darkGray, 2));
		paramBetaBranches.setText("4");
		paramBetaBranches.setBackground(Color.darkGray);
		paramBetaBranches.setForeground(Color.gray);
		paramBetaBranches.setBorder(javax.swing.BorderFactory.createLineBorder(Color.darkGray, 2));
		JLabel l90 = GUIUtils.prepareLabel("Parameters:");
		l90.setHorizontalAlignment(SwingConstants.LEFT);
		JLabel l91 = GUIUtils.prepareLabel("alfa:");
		JLabel l92 = GUIUtils.prepareLabel("beta:");
		l90.setBounds( 0,  0, 200, 20);
		l91.setBounds(10, 25,  20, 20);
		l92.setBounds(10, 50,  20, 20);
		paramAlphaBranches.setBounds(40, 25, 75, 20);
		paramBetaBranches.setBounds (40, 50, 75, 20);
		panelParaBranchesBeta.setLayout(null);
		panelParaBranchesBeta.add(l90);
		panelParaBranchesBeta.add(paramAlphaBranches);
		panelParaBranchesBeta.add(paramBetaBranches);
		panelParaBranchesBeta.add(l91);
		panelParaBranchesBeta.add(l92);

		paramAlphaBranches.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (paramAlphaBranches.getText().length() > 0 && paramBetaBranches.getText().length() > 0) {
					Double alpha = Double.parseDouble(paramAlphaBranches.getText());
					Double beta = Double.parseDouble(paramBetaBranches.getText());
					if (alpha >= 0 && beta >= 0) {
						ProbabilityDistribution p = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
						distr.setDistribution(p);
						distr.repaint();
					} else {
						paramAlphaBranches.setText("0");
						paramBetaBranches.setText("0");
					}
				} else {
					paramAlphaBranches.setText("2");
					paramBetaBranches.setText("4");
				}
			}
		});
		paramBetaBranches.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if (paramAlphaBranches.getText().length() > 0 && paramBetaBranches.getText().length() > 0) {
					Double alpha = Double.parseDouble(paramAlphaBranches.getText());
					Double beta = Double.parseDouble(paramBetaBranches.getText());
					if (alpha >= 0 && beta >= 0) {
						ProbabilityDistribution p = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
						distr.setDistribution(p);
						distr.repaint();
					} else {
						paramAlphaBranches.setText("0");
						paramBetaBranches.setText("0");
					}
				} else {
					paramAlphaBranches.setText("2");
					paramBetaBranches.setText("4");
				}
			}
		});

		return panelParaBranches;
	}


	private void updateDistribution(DistributionViewer distr, Integer maxVal, ProbabilityDistribution distribution) {
		try {
			if (maxVal > 2) {
				distr.setMaxValue(maxVal);
				distr.setDistribution(distribution);
				distr.repaint();
			}
		} catch(NumberFormatException e) {

		}
	}


	private void updateDistribution(DistributionViewer distr, Integer maxVal) {
		try {
			if (maxVal > 2) {
				distr.setMaxValue(maxVal);
				distr.repaint();
			}
		} catch(NumberFormatException e) {

		}
	}


	private void comboANDBranchesChanged(ActionEvent evt) {
		String distrib = (String)comboANDBranches.getSelectedItem();
		ProbabilityDistribution distr = null;
		CardLayout toShow = (CardLayout)panelANDParaBranches.getLayout();
		if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM))) {
			toShow.show(panelANDParaBranches, ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM));
			distr = ProbabilityDistribution.uniformDistributionFactory();
		} else if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA))) {
			toShow.show(panelANDParaBranches, ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA));
			Double alpha = Double.parseDouble(paramANDAlphaBranches.getText());
			Double beta = Double.parseDouble(paramANDBetaBranches.getText());
			distr = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
		} else if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL))) {
			toShow.show(panelANDParaBranches, ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL));
			distr = ProbabilityDistribution.normalDistributionFactory();
		}
		updateDistribution(distribANDBranches, sliderANDBranches.getValue(), distr);
	}


	private void comboANDSortingChanged(ActionEvent evt) {
		String distrib = (String)comboANDSorting.getSelectedItem();
		ProbabilityDistribution distr = null;
		CardLayout toShow = (CardLayout)panelANDParaSorting.getLayout();
		if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM))) {
			toShow.show(panelANDParaSorting, ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM));
			distr = ProbabilityDistribution.uniformDistributionFactory();
		} else if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA))) {
			toShow.show(panelANDParaSorting, ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA));
			Double alpha = Double.parseDouble(paramANDAlphaSorting.getText());
			Double beta = Double.parseDouble(paramANDBetaSorting.getText());
			distr = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
		} else if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL))) {
			toShow.show(panelANDParaSorting, ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL));
			distr = ProbabilityDistribution.normalDistributionFactory();
		}
		updateDistribution(distribANDSorting, sliderANDBranches.getValue(), distr);
	}


	private void comboXORBranchesChanged(ActionEvent evt) {
		String distrib = (String)comboXORBranches.getSelectedItem();
		ProbabilityDistribution distr = null;
		CardLayout toShow = (CardLayout)panelXORParaBranches.getLayout();
		if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM))) {
			toShow.show(panelXORParaBranches, ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM));
			distr = ProbabilityDistribution.uniformDistributionFactory();
		} else if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA))) {
			toShow.show(panelXORParaBranches, ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA));
			Double alpha = Double.parseDouble(paramXORAlphaBranches.getText());
			Double beta = Double.parseDouble(paramXORBetaBranches.getText());
			distr = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
		} else if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL))) {
			toShow.show(panelXORParaBranches, ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL));
			distr = ProbabilityDistribution.normalDistributionFactory();
		}
		updateDistribution(distribXORBranches, sliderXORBranches.getValue(), distr);
	}


	private void comboXORSortingChanged(ActionEvent evt) {
		String distrib = (String)comboXORSorting.getSelectedItem();
		ProbabilityDistribution distr = null;
		CardLayout toShow = (CardLayout)panelXORParaSorting.getLayout();
		if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM))) {
			toShow.show(panelXORParaSorting, ProbabilityDistribution.distributionToName(DISTRIBUTION.UNIFORM));
			distr = ProbabilityDistribution.uniformDistributionFactory();
		} else if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA))) {
			toShow.show(panelXORParaSorting, ProbabilityDistribution.distributionToName(DISTRIBUTION.BETA));
			Double alpha = Double.parseDouble(paramXORAlphaSorting.getText());
			Double beta = Double.parseDouble(paramXORBetaSorting.getText());
			distr = ProbabilityDistribution.betaDistributionFactory(alpha, beta);
		} else if (distrib.equals(ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL))) {
			toShow.show(panelXORParaSorting, ProbabilityDistribution.distributionToName(DISTRIBUTION.NORMAL));
			distr = ProbabilityDistribution.normalDistributionFactory();
		}
		updateDistribution(distribXORSorting, sliderXORBranches.getValue(), distr);
	}
}
