package org.processmining.plugins.plg.gui.widget;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JComponent;
import javax.swing.JFrame;

import org.processmining.plugins.plg.logic.models.distributions.ProbabilityDistribution;

/**
 * This widget is used for showing the current distribution behaviour
 *
 * @author Andrea Burattin
 */
public class DistributionViewer extends JComponent {

	private static final long serialVersionUID = -1606579675680123284L;
	private int minValue = 2;
	private int maxValue = 10;
	protected ProbabilityDistribution distribution = ProbabilityDistribution.normalDistributionFactory();

	
	/**
	 * Get the value of distribution
	 *
	 * @return the value of distribution
	 */
	public ProbabilityDistribution getDistribution() {
		return distribution;
	}


	/**
	 * Set the value of distribution
	 *
	 * @param distribution new value of distribution
	 */
	public void setDistribution(ProbabilityDistribution distribution) {
		this.distribution = distribution;
	}


	/**
	 * Get the value of maxValue
	 *
	 * @return the value of maxValue
	 */
	public int getMaxValue() {
		return maxValue;
	}


	/**
	 * Set the value of maxValue
	 *
	 * @param maxValue new value of maxValue
	 */
	public void setMaxValue(int maxValue) {
		this.maxValue = maxValue;
	}


	/**
	 * Get the value of minValue
	 *
	 * @return the value of minValue
	 */
	public int getMinValue() {
		return minValue;
	}


	/**
	 * Set the value of minValue
	 *
	 * @param minValue new value of minValue
	 */
	public void setMinValue(int minValue) {
		this.minValue = minValue;
	}

	
	@SuppressWarnings("cast")
	@Override
	protected void paintComponent(Graphics g) {
		int width = getWidth();
		int height = getHeight();
		Graphics2D gBuf = (Graphics2D)g;
//		gBuf.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		// BACKGROUND
//		gBuf.setColor(Color.WHITE);
//		gBuf.fillRect(0, 0, width, height);

		// MAIN AXES
		int realHeight = height-17;
		int pos1 = width/3;
		int pos2 = (width/3)*2;

		gBuf.setColor(new Color(150,150,150));
		gBuf.fillRect(5, 5, width-10, height-17);
		
		gBuf.setColor(Color.BLACK);
		gBuf.drawRect(5, 5, width-10, height-17);
		gBuf.drawLine(3, 5+realHeight/2, 4, 5+realHeight/2);
		gBuf.drawLine(pos1, 3, pos1, 4);
		gBuf.drawLine(pos2, 3, pos2, 4);

		gBuf.setColor(Color.white);
		for(int i = 6; i < width-10; i+=5) {
			gBuf.drawLine(i, 5+realHeight/2, i+2, 5+realHeight/2);
		}
		for(int i = 7; i < height-17; i+=5) {
			gBuf.drawLine(pos1, i, pos1, i+2);
			gBuf.drawLine(pos2, i, pos2, i+2);
		}


		// INSERT ALL THE TEXTS
		gBuf.setColor(new Color(40,40,40));
		gBuf.setFont(gBuf.getFont().deriveFont(10.0f));
		gBuf.drawString(new Integer(minValue).toString(), 5, height-2);
		if (maxValue < 10) {
			gBuf.drawString(new Integer(maxValue).toString(), width-10, height-2);
		} else if (maxValue < 100) {
			gBuf.drawString(new Integer(maxValue).toString(), width-15, height-2);
		} else {
			gBuf.drawString(new Integer(maxValue).toString(), width-20, height-2);
		}
		int delta = maxValue - minValue;
		if (maxValue >= 10) {
			gBuf.drawString(new Integer(new Double(Math.ceil(delta/3.)).intValue()).toString(), (width/3)-5, height-2);
			gBuf.drawString(new Integer(new Double(Math.ceil(delta/3.*2)).intValue()).toString(), (width/3*2)-5, height-2);
		}

		// BASIC VALUES
		int newWidth = width - 12;
		double diffHeight = height - 17;
		int prevX = -10;
		int prevY = -10;
		int size = maxValue-minValue+1;
		if (maxValue > 1000) {
			size = 1000-minValue+1;
		}

		Double[] plot = distribution.getValuesForPlotting(size+1);
		
		// draw the lines
		for (int i = 0; i < size+1; i++) {
			int x = Math.round(5.f + (((float)newWidth / (float)size) * (float)(i)));
			double v = plot[i];
			int y = (int)Math.round(diffHeight * v) - 5;

			if (prevY > -10) {
				gBuf.setColor(Color.GRAY);
				drawThickLine(gBuf, prevX+1, height-17-prevY, x+1, height-17-y, 2);
			}
			prevX = x;
			prevY = y;
		}
		
		// redraw the basic grid
		gBuf.setColor(Color.black);
		gBuf.drawRect(5, 5, width-10, height-17);
		
		// draw the points
		for (int i = 0; i < size+1; i++) {
			int x = Math.round(5.f + (((float)newWidth / (float)size) * (float)(i)));
			double v = plot[i];
			int y = (int)Math.round(diffHeight * v) - 5;
			
			gBuf.setColor(new Color(230,0,0));
			gBuf.fillOval(x-1, height-18-y-1, 4, 4);
		}

		gBuf.dispose();
	}
	
	@SuppressWarnings("cast")
	private void drawThickLine(Graphics g, int x1, int y1, int x2, int y2, int thickness) {
		int dX = x2 - x1;
		int dY = y2 - y1;

		double lineLength = Math.sqrt(dX * dX + dY * dY);
		double scale = (double)(thickness) / (2.0 * lineLength);
		
		double ddx = -scale * (double)dY;
		double ddy = scale * (double)dX;
		ddx += (ddx > 0) ? 0.5 : -0.5;
		ddy += (ddy > 0) ? 0.5 : -0.5;
		int dx = (int)ddx;
		int dy = (int)ddy;

		int xPoints[] = new int[4];
		int yPoints[] = new int[4];
		
		xPoints[0] = x1 + dx; yPoints[0] = y1 + dy;
		xPoints[1] = x1 - dx; yPoints[1] = y1 - dy;
		xPoints[2] = x2 - dx; yPoints[2] = y2 - dy;
		xPoints[3] = x2 + dx; yPoints[3] = y2 + dy;
		
		g.fillPolygon(xPoints, yPoints, 4);
	}

	/* Test main
	 */
	public static void main(String[] a) {
		System.out.println("Loading widget...");
		JFrame f = new JFrame("DistributionViewer test");
		f.setSize(200, 130);
		f.setVisible(true);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setLayout(new BorderLayout());
		
		DistributionViewer dv = new DistributionViewer();
		f.add(dv);
		
		System.out.println("Done!");
	}
}
