package org.processmining.plugins.plg.io;

import java.io.File;
import java.io.IOException;

import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.plg.models.PLGProcessCollection;

/** 
 * Class with the export plugin for a collection of processes
 * 
 * @author Andrea Burattin
 */
@Plugin(
	name = "Export PLG Process Collection (.plg)",
	returnLabels = {},
	returnTypes = {},
	parameterLabels = { "Process Collection", "File" },
	userAccessible = true
)
@UIExportPlugin(
	description = "Export PLG Process Collection (.plg)",
	extension = "plg"
)
public class ExportProcessCollection {

	/**
	 * This plugin is used to export a process collection object
	 * 
	 * @param context
	 * @param processCollection
	 * @param file
	 * @throws IOException
	 */
	@PluginVariant(
		variantLabel = "Export PLG Process Collection (.plg)",
		requiredParameterLabels = { 0, 1 }
	) 
	public void exportPLGProcessCollection(
			PluginContext context, 
			PLGProcessCollection processCollection, 
			File file) throws IOException {
		context.log("Export started..."); 
		processCollection.saveCollection(file);
		context.log("Export complete!");
	} 
}
