package org.processmining.plugins.plg.logic.models;


/**
 * This class represents a frame containing a process pattern
 * 
 * With this class is possible to describe the bounds of a process pattern
 * instance, since it contains information on the `head' and the `tail' of the
 * pattern.
 * 
 * @author Andrea Burattin
 */
public class PlgPatternFrame {
	
	private static int globalCounter = 0;
	
	private int id;
	private PlgActivity head;
	private PlgActivity tail;
	private Boolean isLoop;
	
	
	/**
	 * The default constructor of the pattern frame
	 * 
	 * @param head the head of the pattern
	 * @param tail the tail of the pattern
	 */
	public PlgPatternFrame(PlgActivity head, PlgActivity tail) {
		this(head, tail, false);
	}
	
	
	/**
	 * Constructor of the pattern frame
	 * 
	 * @param head the head of the pattern
	 * @param tail the tail of the pattern
	 * @param isLoop if the current frame represent a loop
	 */
	public PlgPatternFrame(PlgActivity head, PlgActivity tail, Boolean isLoop) {
		
		this.head = head;
		this.tail = tail;
		this.id = globalCounter++;
		this.isLoop = isLoop;
		
		head.setFrame(this);
		tail.setFrame(this);
	}
	
	
	/**
	 * Get the head of the pattern
	 * 
	 * @return the head of the pattern
	 */
	public PlgActivity getHead() {
		return head;
	}
	
	
	/**
	 * Set the head of the pattern
	 * 
	 * @param head the new head
	 */
	public void setHead(PlgActivity head) {
		this.head = head;
	}
	
	
	/**
	 * Get the tail of the pattern
	 * 
	 * @return the tail of the pattern
	 */
	public PlgActivity getTail() {
		return tail;
	}
	
	
	/**
	 * Set the tail of the pattern
	 * 
	 * @param tail the new tail
	 */
	public void setTail(PlgActivity tail) {
		this.tail = tail;
	}
	
	
	/**
	 * 
	 * @return
	 */
	public String getID() {
		return new Integer(id).toString();
	}
	
	
	/**
	 * 
	 * @return
	 */
	public Boolean isLoop() {
		return isLoop;
	}
}
